# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Generate SensorGrids of radial directions around positions from the floors of rooms.
_
This type of sensor grid is particularly helpful for studies of multiple view
directions, such as imageless glare studies.
_
The names of the grids will be the same as the rooms that they came from.
-

    Args:
        _rooms: A list of honeybee Rooms for which sensor grids will be generated.
            This can also be an entire Honeybee Model from which Rooms will
            be extracted.
        _grid_size: Number for the size of the grid cells.
        _dist_floor_: Number for the distance to move points from the floors of
            the input rooms. (Default: 1.2 meters).
        _dir_count_: A positive integer for the number of radial directions to be
            generated around each position. (Default: 8).
        _start_vec_: A Vector3D to set the start direction of the generated directions.
            This can be used to orient the resulting sensors to specific parts
            of the scene. It can also change the elevation of the resulting
            directions since this start vector will always be rotated in the
            XY plane to generate the resulting directions. (Default: (0, -1, 0)).
        wall_offset_: A number for the distance at which sensors close to walls
            should be removed.

    Returns:
        grid: A SensorGrid object that can be used in a grid-based recipe.
        points: The points that are at the center of each circle. These align with
            the vecs output below and can be visualized with the native
            Grasshopper vector display component.
        vecs: The vectors for the directions of each sensor. These align with
            the points output above and can be visualized with the native
            Grasshopper vector display component.
        mesh: Analysis mesh that can be passed to the 'Spatial Heatmap' component.
"""

ghenv.Component.Name = 'HB Radial Grid from Rooms'
ghenv.Component.NickName = 'RadialGridRooms'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import the ladybug_geometry dependencies
    from ladybug_geometry.geometry3d.pointvector import Vector3D, Point3D
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))

try:  # import the core honeybee dependencies
    from honeybee.model import Model
    from honeybee.room import Room
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.config import conversion_to_meters
    from ladybug_rhino.togeometry import to_vector3d
    from ladybug_rhino.fromgeometry import from_mesh3d, from_point3d, from_vector3d
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set defaults for any blank inputs and process the quad_only_
    _dist_floor_ = 1.2 / conversion_to_meters() if _dist_floor_ is None else _dist_floor_
    wall_offset = 0 if wall_offset_ is None else wall_offset_
    dir_count = 8 if _dir_count_ is None else _dir_count_
    try:
        st_vec = to_vector3d(_start_vec_)
    except AttributeError:
        st_vec = Vector3D(0, -1, 0)

    # gather all of the rooms
    rooms = []
    for obj in _rooms:
        if isinstance(obj, Model):
            rooms.extend(obj.rooms)
        elif isinstance(obj, Room):
            rooms.append(obj)
        else:
            raise TypeError('Expected Honeybee Room or Model. Got {}.'.format(type(obj)))

    # create lists to be filled with content
    grid, points, vecs, mesh = [], [], [], []
    # loop through the rooms and create the grids
    for room in rooms:
        s_grid = room.properties.radiance.generate_sensor_grid_radial(
            _grid_size, offset=_dist_floor_, remove_out=True, wall_offset=wall_offset,
            dir_count=dir_count, start_vector=st_vec)

        if s_grid is not None:
            grid.append(s_grid)
            sensors = s_grid.sensors
            base_points = [from_point3d(Point3D(*sen.pos)) for sen in sensors]
            base_vecs = [from_vector3d(Vector3D(*sen.dir)) for sen in sensors]
            points.append(base_points)
            vecs.append(base_vecs)
            lb_mesh = s_grid.mesh
            if lb_mesh is not None:
                mesh.append(from_mesh3d(lb_mesh))
            else:
                mesh.append(None)

    # convert the lists of points to data trees
    points = list_to_data_tree(points)
    vecs = list_to_data_tree(vecs)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Run a Sky View (SV) study for a Honeybee model.
_
Sky View is defined as the percent of the sky dome seen by a surface. These can
be computed either using a uniform (default) sky or a cloudy sky.
_
Note that computing cloudy Sky View for a vertically-oriented geometry (horizontal
sensor direction) will yield Vertical Sky Component (VSC) as described by the UK
Building Research Establishment (BRE). VSC is defined as the ratio of cloudy sky
illuminance falling on a vertical wall to the simultaneous horizontal illuminance
under an unobstructed sky [Littlefair, 1991].
_
Also note that this recipe still respects the transparency of objects, reducing
the percentage of the sky visible through a certain geometry by the transmittance
of that geometry.

-
    Args:
        _model: A Honeybee Model for which Sky View or Wky Exposure will be simulated.
            Note that this model should have grids assigned to it in order
            to produce meaningful results.
        cloudy_sky_: A boolean to note whether a uniform sky should be used  (False) or
            a cloudy overcast sky (True). (Default: False).
        grid_filter_: Text for a grid identifer or a pattern to filter the sensor grids of
            the model that are simulated. For instance, `first_floor_*` will simulate
            only the sensor grids that have an identifier that starts with
            `first_floor_`. By default, all grids in the model will be simulated.
        radiance_par_: Text for the radiance parameters to be used for ray
            tracing. (Default: -ab 2 -aa 0.1 -ad 2048 -ar 64).
        run_settings_: Settings from the "HB Recipe Settings" component that specify
            how the recipe should be run. This can also be a text string of
            recipe settings.
        _run: Set to True to run the recipe and get results.

    Returns:
        report: Reports, errors, warnings, etc.
        results: Numbers for the sky view or sky exposure at each sensor. These can be plugged
            into the "LB Spatial Heatmap" component along with meshes of the
            sensor grids to visualize results. Values are in percent (between 0
            and 100).
"""

ghenv.Component.Name = 'HB Sky View'
ghenv.Component.NickName = 'SkyView'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from lbt_recipes.recipe import Recipe
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run:
    # create the recipe and set the input arguments
    recipe = Recipe('sky-view')
    recipe.input_value_by_name('model', _model)
    recipe.input_value_by_name('grid-filter', grid_filter_)
    recipe.input_value_by_name('cloudy-sky', cloudy_sky_)
    recipe.input_value_by_name('radiance-parameters', radiance_par_)

    # run the recipe
    silent = True if _run > 1 else False
    project_folder = recipe.run(run_settings_, radiance_check=True, silent=silent)

    # load the results
    try:
        results = recipe_result(recipe.output_value_by_name('results', project_folder))
    except Exception:
        raise Exception(recipe.failure_message(project_folder))



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Run a point-in-time grid-based study for a Honeybee model.
_
Point-in-time recipes require a sky and can output illuminance, irradiance,
luminance or radiance.

-
    Args:
        _model: A Honeybee Model for which a point-in-time grid-based study will be run.
            Note that this model should have grids assigned to it in order
            to produce meaningfule results.
        _sky: A Radiance sky from any of the sky components under the "Light Sources" tab.
            Skies can be either CIE, ClimateBased/Custom, or for a specific
            Illuminance/Irradiance. This input can also just be a text definition
            of a sky's paramters. Examples include:
                * cie 21 Mar 9:00 -lat 41.78 -lon -87.75 -tz 5 -type 0
                * climate-based 21 Jun 12:00 -lat 41.78 -lon -87.75 -tz 5 -dni 800 -dhi 120
                * irradiance 0
        _metric_: Either an integer or the full name of a point-in-time metric to be computed
            by the recipe. (Default: illuminance). Choose from the following:
                * 0 = illuminance
                * 1 = irradiance
                * 2 = luminance
                * 3 = radiance
        grid_filter_: Text for a grid identifer or a pattern to filter the sensor grids of
            the model that are simulated. For instance, `first_floor_*` will simulate
            only the sensor grids that have an identifier that starts with
            `first_floor_`. By default, all grids in the model will be simulated.
        radiance_par_: Text for the radiance parameters to be used for ray
            tracing. (Default: -ab 2 -aa 0.1 -ad 2048 -ar 64).
        run_settings_: Settings from the "HB Recipe Settings" component that specify
            how the recipe should be run. This can also be a text string of
            recipe settings.
        _run: Set to True to run the recipe and get results. This input can also be
            the integer "2" to run the recipe silently.

    Returns:
        report: Reports, errors, warnings, etc.
        results: Numbers for the point-in-time value at each sensor. Values are in the
            standard SI units of the requested input metric. These can be plugged
            into the "LB Spatial Heatmap" component along with meshes of the
            sensor grids to visualize results.
                * illuminance = lux (aka. lm/m2)
                * irradiance = W/m2
                * luminance = cd/m2 (aka. lm/m2-sr)
                * radiance = W/m2-sr
"""

ghenv.Component.Name = 'HB Point-In-Time Grid-Based'
ghenv.Component.NickName = 'PITGrid'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:
    from lbt_recipes.recipe import Recipe
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run:
    # create the recipe and set the input arguments
    recipe = Recipe('point-in-time-grid')
    recipe.input_value_by_name('model', _model)
    recipe.input_value_by_name('sky', _sky)
    recipe.input_value_by_name('metric', _metric_)
    recipe.input_value_by_name('grid-filter', grid_filter_)
    recipe.input_value_by_name('radiance-parameters', radiance_par_)

    # run the recipe
    silent = True if _run > 1 else False
    project_folder = recipe.run(run_settings_, radiance_check=True, silent=silent)

    # load the results
    try:
        results = recipe_result(recipe.output_value_by_name('results', project_folder))
    except Exception:
        raise Exception(recipe.failure_message(project_folder))



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Generate SensorGrid objects from exterior Apertures.
_
These SensorGrids can be used in any grid-based recipe and are particularly useful
for irradiance studies that evaluate solar gain of buildings, such as peak solar
irradiance studies.
-

    Args:
        _hb_objs: A list of honeybee Faces or Rooms for which sensor grids will be
            generated. This can also be an entire Honeybee Model.
        _grid_size: Number for the size of the grid cells.
        _offset_: Number for the distance to move points from the base geometry.
            Positive numbers indicate an offset towards the exterior while
            negative numbers indicate an offset towards the interior, essentially
            modeling the value of trasnmitted sun through the glass. The default
            is 0.1 meters.
        _ap_type_: Text or an integer to specify the type of aperture that will be used to
            generate grids. Choose from the following. (Default: All).
                * 1 - Window
                * 2 - Skylight
                * 3 - All
        quad_only_: Boolean to note whether meshing should be done using Rhino's
            defaults (False), which fills the entire aperture geometry to the edges
            with both quad and tringulated faces, or a mesh with only quad
            faces should be generated. (Default: False).

    Returns:
        grid: A SensorGrid object that can be used in a grid-based recipe.
        points: The points that are at the center of each grid cell.
        mesh: Analysis mesh that can be passed to the 'Spatial Heatmap' component.
"""

ghenv.Component.Name = 'HB Sensor Grid from Apertures'
ghenv.Component.NickName = 'GridApertures'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import the ladybug_geometry dependencies
    from ladybug_geometry.geometry3d.mesh import Mesh3D
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))

try:  # import the core honeybee dependencies
    from honeybee.model import Model
    from honeybee.room import Room
    from honeybee.face import Face
    from honeybee.aperture import Aperture
    from honeybee.boundarycondition import Outdoors
    from honeybee.facetype import Floor, Wall, RoofCeiling
    from honeybee.typing import clean_rad_string, clean_and_id_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.sensorgrid import SensorGrid
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.config import conversion_to_meters
    from ladybug_rhino.togeometry import to_joined_gridded_mesh3d
    from ladybug_rhino.fromgeometry import from_mesh3d, from_point3d, from_face3d
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


APERTURE_TYPES = {
    '1': Wall,
    '2': RoofCeiling,
    '3': (Wall, RoofCeiling, Floor),
    'Window': Wall,
    'Skylight': RoofCeiling,
    'All': (Wall, RoofCeiling, Floor)
}


if all_required_inputs(ghenv.Component):
    # set defaults for any blank inputs
    _offset_ = 0.1 / conversion_to_meters() if _offset_ is None else _offset_
    ft = APERTURE_TYPES[_ap_type_.title()] if _ap_type_ is not None \
        else (Wall, RoofCeiling, Floor)

    # collect all of the relevant apertures
    apertures = []
    for obj in _hb_objs:
        if isinstance(obj, (Model, Room)):
            for face in obj.faces:
                if isinstance(face.boundary_condition, Outdoors) and isinstance(face.type, ft):
                    apertures.extend(face.apertures)
        elif isinstance(obj, Face):
            if isinstance(obj.boundary_condition, Outdoors) and isinstance(obj.type, ft):
                apertures.extend(obj.apertures)
        elif isinstance(obj, Aperture):
            if obj.has_parent:
                face = obj.parent
                if isinstance(face.boundary_condition, Outdoors) and isinstance(face.type, ft):
                    apertures.append(obj)
            else:
                apertures.append(obj)
        else:
            raise TypeError(
                'Expected Honeybee Aperture, Face, Room or Model. Got {}.'.format(type(obj)))

    # greneate the meshes and grids from the faces
    if len(apertures) != 0:
        # create the gridded ladybug Mesh3D
        f_geos = [ap.geometry for ap in apertures]
        if quad_only_:  # use Ladybug's built-in meshing methods
            lb_meshes = []
            for geo in f_geos:
                try:
                    lb_meshes.append(geo.mesh_grid(_grid_size, offset=_offset_))
                except AssertionError:
                    continue
            if len(lb_meshes) == 0:
                lb_mesh = None
            else:
                lb_mesh = lb_meshes[0] if len(lb_meshes) == 1 else \
                    Mesh3D.join_meshes(lb_meshes)
        else:  # use Rhino's default meshing
            rh_faces = [from_face3d(face) for face in f_geos]
            lb_mesh = to_joined_gridded_mesh3d(rh_faces, _grid_size, _offset_)

        if lb_mesh is not None:
            # extract positions and directions from the mesh
            mesh = from_mesh3d(lb_mesh)
            points = [from_point3d(pt) for pt in lb_mesh.face_centroids]
            base_poss = [(pt.x, pt.y, pt.z) for pt in lb_mesh.face_centroids]
            base_dirs = [(vec.x, vec.y, vec.z) for vec in lb_mesh.face_normals]

            # create the sensor grid
            f_nm = 'Windows'
            if isinstance(ft, tuple):
                f_nm = 'Apertures' 
            elif ft is RoofCeiling:
                f_nm = 'Skylights'
            g_name = clean_rad_string('{}_Exterior{}'.format(_hb_objs[0].display_name, f_nm)) \
                if len(_hb_objs) == 1 else clean_and_id_rad_string('Exterior{}'.format(f_nm))
            grid = SensorGrid.from_position_and_direction(g_name, base_poss, base_dirs)
            grid.mesh = lb_mesh



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Interpolate or extrapolate a High Dynamic Range (HDR) image file from another
HDR image file.
_
Recommended use is to extract 180 FOV (-vh 180 -vv 180) angular or hemispherical
HDR images from a 360 FOV (-vh 360 -vv 360) angular HDR image. Alternatively,
conversions between 180 FOV angular and hemispherical HDR images can be made.
-

    Args:
        _view: A view to interpolate or extrapolate into a new HDR. The "HB View" 
            component can be used to create an input view and it must
            have the same position as that use to make the _hdr.
        _hdr: Path to a High Dynamic Range (HDR) image file from which to
            interpolate or extrapolate.
        _resolution_: An integer for the dimension of the output image in pixels.
            If extracting a 180 FOV angular or hemispherical HDR image from a
            360 FOV HDR image, the default resolution is 1/3 of the resolution of
            _hdr. If converting between 180 FOV angular or hemispherical HDR 
            images, the default resolution is that of _hdr.

    Returns:
        hdr: Path to the resulting HDR image file.
"""

ghenv.Component.Name = 'HB Extract HDR'
ghenv.Component.NickName = 'ExtractHDR'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '0'

import os
import subprocess
import re

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.pinterp import Pinterp
    from honeybee_radiance_command.ra_xyze import Ra_xyze
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.config import folders as rad_folders
    from honeybee_radiance.view import View
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs, give_warning
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

# check the Radiance date of the installed radiance
try:  # import lbt_recipes dependencies
    from lbt_recipes.version import check_radiance_date
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))
check_radiance_date()


def check_view_hdr(hdr_path):
    """Check if the header of the HDR image contains a view (VIEW=).
    
    A ValueError is raised if the image does not contain a valid view.
    A ValueError is raised if the view type is not -vta or -vth.
    
    Args:
        hdr_path: The path to an HDR image file.
    """
    # set hdr_view to None
    hdr_view = None
    
    # read hdr image and search for a valid view
    with open(hdr_path, 'r') as hdr_file:
        for lineCount, line in enumerate(hdr_file):
            if lineCount < 200:
                low_line = line.lower()
                if not low_line.startswith('\t'):
                    if low_line.startswith('view='):
                        hdr_view = View.from_string('hdr_view', line)
            else:  # no need to check the rest of the document
                break
    if not hdr_view:
        raise ValueError(
            'Connected _hdr image does not contain a valid view in the header.\n'
            'Note that indented views in the header will be ignored by pinterp.')
    if not hdr_view.type in ('a', 'h'):
        msg = 'Expected view type -vta or -vth in _hdr. Got view type -vt{}.'
        raise ValueError(msg.format(hdr_view.type))
    return hdr_view


def check_view_points(view, hdr_view):
    """Check if view points of output view and input HDR are matching.
    
    A ValueError is raised if the view points are not matching.
    
    Args:
        view: A Honeybee Radiance View to extract.
        hdr_view: A Honeybee Radiance View from the input HDR.
    """
    if not view.position == hdr_view.position:
        msg = 'View points of _view and _hdr are not matching.\n' \
        'Got _view = {} and _hdr = {}.'
        raise ValueError(msg.format(view.position, hdr_view.position))


def check_resolution(hdr_path, resolution, view, hdr_view):
    """Check the resolution of the output HDR as well as the input HDR.
    
    A warning is raised if the HDR image dimensions are not square. A warning is
    raised if the resolution is larger than one third of the HDR image
    resolution if converting a 360 FOV HDR to 180 FOV HDR. A warning is raised 
    if the output resolution is larger than the input resolution.
    
    Args:
        hdr_path: The path to an HDR image file.
        resolution: The resolution of the extracted view from hdr_path.
        view: A Honeybee Radiance View to extract.
        hdr_view: A Honeybee Radiance View from the input HDR.
    """
    # get the path the the getinfo command
    getinfo_exe = os.path.join(rad_folders.radbin_path, 'getinfo.exe') if \
        os.name == 'nt' else os.path.join(rad_folders.radbin_path, 'getinfo')
    
    # run the getinfo command in a manner that lets us obtain the result
    cmds = [getinfo_exe, '-d', hdr_path]
    use_shell = True if os.name == 'nt' else False
    process = subprocess.Popen(cmds, stdout=subprocess.PIPE, shell=use_shell)
    stdout = process.communicate()
    img_dim = stdout[0]

    def get_dimensions(img_dim):
        dimensions = []
        for d in ['+X', '-Y']:
            regex = r'\%s\s+(\d+)' % d
            matches = re.finditer(regex, img_dim, re.MULTILINE)
            dim = next(matches).groups()[0]
            dimensions.append(int(dim))
        return dimensions
    # check the X and Y dimensions of the image
    hdr_x, hdr_y = get_dimensions(img_dim)
 
    if hdr_x == hdr_y: 
        hdr_resolution = hdr_x = hdr_y
    else:
        msg = 'It is recommended that image dimensions of _hdr are square.\n' \
            'Got {} x {}.'
        give_warning(ghenv.Component, msg.format(hdr_x, hdr_y))
    
    # check resolution ratio of output image / input image
    if hdr_view.h_size == 360 and hdr_view.v_size == 360:
        if resolution is None: 
            resolution = hdr_resolution / 3
        if resolution > hdr_resolution / 3:
            msg = 'Recommended _resolution_ is one third or less of the _hdr resolution. \n' \
                'Got {} for _resolution_ and {} for _hdr. Recommended _resolution_ \n' \
                'is {} or lower.'
            give_warning(ghenv.Component, msg.format(resolution, hdr_resolution, 
                         int(hdr_resolution / 3)))
    else:
        if resolution is None: 
            resolution = hdr_resolution
        if resolution > hdr_resolution:
            msg = 'Output image resolution ({}) is larger than input image \n' \
                'resolution ({}). It is recommended that _resolution_ is equal \n' \
                'to or less than input image resolution.'
            give_warning(ghenv.Component, msg.format(resolution, hdr_resolution))
    return resolution


if all_required_inputs(ghenv.Component):
    # check if _view is Honeybee Radiance View
    assert isinstance(_view, View), \
        'Expected Honeybee Radiance View in _view. Got {}.'.format(type(_view))
    
    # check if header contains a view
    hdr_view = check_view_hdr(_hdr)
    
    # check view points
    check_view_points(_view, hdr_view)
    
    # check resolution
    resolution = check_resolution(_hdr, _resolution_, _view, hdr_view)
    
    # set up the paths for the various files used in translation
    img_dir = os.path.dirname(_hdr)
    input_image = os.path.basename(_hdr)
    commands = []
    
    # add the command to include exposure in the pixels
    expos_image = input_image.lower().replace('.hdr', '_e.hdr')
    ra_xyze = Ra_xyze(input=input_image, output=expos_image)
    ra_xyze.options.r = True
    ra_xyze.options.o = True
    commands.append(ra_xyze)
    
    # add the command to extract a view (HDR)
    view_identifier = _view.identifier
    view = os.path.basename(_view.to_file(img_dir))
    pinterp_image = input_image.lower().replace('.hdr', '_{}.hdr'.format(view_identifier))
    pinterp = Pinterp(output=pinterp_image, view=view, image=expos_image,
                      zspec=1)
    pinterp.options.x = resolution
    pinterp.options.y = resolution
    commands.append(pinterp)
    hdr = os.path.join(img_dir, pinterp_image)
    
    # run the commands in series
    env = None
    if rad_folders.env != {}:
        env = rad_folders.env
    env = dict(os.environ, **env) if env else None
    for r_cmd in commands:
        r_cmd.run(env, cwd=img_dir)



# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Get average illuminance or irradiance values over an annual daylight or irradiance
simulation.
_
The _hoys_ input can also be used to filter the data for a particular time period or
hour/timestep of the simulation.

-
    Args:
        _results: An list of annual Radiance result files from either the "HB Annual Daylight"
            or the "HB Annual Irradiance" component (containing the .ill files and
            the sun-up-hours.txt). This can also be just the path to the folder
            containing these result files.
        dyn_sch_: Optional dynamic Aperture Group Schedules from the "HB Aperture Group
            Schedule" component, which will be used to customize the behavior
            of any dyanmic aperture geometry in the output metrics. If unsupplied,
            all dynamic aperture groups will be in their default state in for
            the output metrics.
        _hoys_: An optional numbers or list of numbers to select the hours of the year (HOYs)
            for which results will be computed. These HOYs can be obtained from the
            "LB Calculate HOY" or the "LB Analysis Period" components. If None, all
            hours of the results will be used.
        median_: Set to True to get the median values instead of the average. The median
            values can only be calculated for a results folder from the
            "HB Annual Daylight" component. (Default: False).
        grid_filter_: The name of a grid or a pattern to filter the grids. For instance,
            first_floor_* will simulate only the sensor grids that have an
            identifier that starts with first_floor_. By default all the grids
            will be processed.

    Returns:
        report: Reports, errors, warnings, etc.
        values: Average illuminance or irradiance valules for each sensor in lux or W/m2.
            Each value is for a different sensor of the grid. These can be plugged
            into the "LB Spatial Heatmap" component along with meshes of the sensor
            grids to visualize results.
"""

ghenv.Component.Name = 'HB Annual Average Values'
ghenv.Component.NickName = 'AvgValues'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os
import subprocess

try:
    from ladybug.futil import write_to_file
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee.config import folders
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_radiance.postprocess.annualdaylight import _process_input_folder
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from honeybee_radiance_postprocess.dynamic import DynamicSchedule
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from pollination_handlers.outputs.helper import read_sensor_grid_result
except ImportError as e:
    raise ImportError('\nFailed to import pollination_handlers:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree, \
        give_warning
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


def parse_sun_up_hours(sun_up_hours, hoys, timestep):
    """Parse the sun-up hours from the result file .txt file.

    Args:
        sun_up_hours: A list of integers for the sun-up hours.
        hoys: A list of 8760 * timestep values for the hoys to select. If an empty
            list is passed, None will be returned.
        timestep: Integer for the timestep of the analysis.
    """
    if len(hoys) != 0:
        schedule = [False] * (8760 * timestep)
        for hr in hoys:
            schedule[int(hr * timestep)] = True
        su_pattern = [schedule[int(h * timestep)] for h in sun_up_hours]
        return su_pattern


def average_values(ill_file, su_pattern, full_len):
    """Compute average values for a given result file."""
    avg_vals = []
    with open(ill_file) as results:
        if su_pattern is None:  # no HOY filter on results
            for pt_res in results:
                values = [float(r) for r in pt_res.split()]
                total_val = sum(values)
                avg_vals.append(total_val / full_len)
        else: 
            for pt_res in results:
                values = [float(r) for r, is_hoy in zip(pt_res.split(), su_pattern) if is_hoy]
                total_val = sum(values)
                try:
                    avg_vals.append(total_val / full_len)
                except ZeroDivisionError:
                    avg_vals.append(0)
    return avg_vals


if all_required_inputs(ghenv.Component):
    # set up the default values
    median_ = False if median_ is None else median_
    grid_filter_ = '*' if grid_filter_ is None else grid_filter_
    res_folder = os.path.dirname(_results[0]) if os.path.isfile(_results[0]) \
        else _results[0]

    # check to see if results use the newer numpy arrays
    if os.path.isdir(os.path.join(res_folder, '__static_apertures__')) or \
        os.path.isfile(os.path.join(res_folder, 'grid_states.json')):
        res_type = 'average' if median_ is False else 'median'
        cmds = [folders.python_exe_path, '-m', 'honeybee_radiance_postprocess',
                'post-process', '{}-values'.format(res_type), res_folder, '-sf',
                'metrics']
        if len(_hoys_) != 0:
            hoys_str = '\n'.join(str(h) for h in _hoys_)
            hoys_file = os.path.join(res_folder, 'hoys.txt')
            write_to_file(hoys_file, hoys_str)
            cmds.extend(['--hoys-file', hoys_file])
        if grid_filter_ != '*':
            cmds.extend(['--grids-filter', grid_filter_])
        if len(dyn_sch_) != 0:
            if os.path.isfile(os.path.join(res_folder, 'grid_states.json')):
                dyn_sch = dyn_sch_[0] if isinstance(dyn_sch_[0], DynamicSchedule) else \
                    DynamicSchedule.from_group_schedules(dyn_sch_)
                dyn_sch_file = dyn_sch.to_json(folder=res_folder)
                cmds.extend(['--states', dyn_sch_file])
            else:
                msg = 'No dynamic aperture groups were found in the Model.\n' \
                    'The input dynamic schedules will be ignored.'
                print(msg)
                give_warning(ghenv.Component, msg)
        use_shell = True if os.name == 'nt' else False
        custom_env = os.environ.copy()
        custom_env['PYTHONHOME'] = ''
        process = subprocess.Popen(
            cmds, cwd=res_folder, shell=use_shell, env=custom_env,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout = process.communicate()  # wait for the process to finish
        if stdout[-1] != '':
            print(stdout[-1])
            raise ValueError('Failed to compute {} values.'.format(res_type))
        res_dir = os.path.join(res_folder, 'metrics', '{}_values'.format(res_type))
        if os.path.isdir(res_dir):
            values = read_sensor_grid_result(res_dir, res_type,'full_id', False)
            values = list_to_data_tree(values)

    else:
        if len(dyn_sch_) != 0:
            msg = 'Dynamic Schedules are currently only supported for Annual Daylight ' \
                'simulations.\nThe input schedules will be ignored.'
            print(msg)
            give_warning(ghenv.Component, msg)
        if median_:
            raise ValueError('The median values can only be calculated for a '
                             'results folder from the "HB Annual Daylight" '
                             'component.')
        # extract the timestep if it exists
        timestep = 1
        tstep_file = os.path.join(res_folder, 'timestep.txt')
        if os.path.isfile(tstep_file):
            with open(tstep_file) as tf:
                timestep = int(tf.readline())
        full_len = 8760 * timestep if len(_hoys_) == 0 else len(_hoys_)

        # parse the sun-up-hours
        grids, sun_up_hours = _process_input_folder(res_folder, grid_filter_)
        su_pattern = parse_sun_up_hours(sun_up_hours, _hoys_, timestep)

        # compute the average values
        values = []
        for grid_info in grids:
            ill_file = os.path.join(res_folder, '%s.ill' % grid_info['full_id'])
            dgp_file = os.path.join(res_folder, '%s.dgp' % grid_info['full_id'])
            if os.path.isfile(dgp_file):
                avg = average_values(dgp_file, su_pattern, full_len)
            else:
                avg = average_values(ill_file, su_pattern, full_len)
            values.append(avg)
        values = list_to_data_tree(values)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Apply ModifierSet to Honeybee Rooms.
-

    Args:
        _rooms: Honeybee Rooms to which the input _mod_set should be assigned.
            This can also be a Honeybee Model for which all Rooms
            will be assigned the ModifierSet.
        _mod_set: A Honeybee ModifierSet to be applied to the input _room.
            This can also be text for a modifier set to be looked up in the
            modifier set library.

    Returns:
        rooms: The input Rooms with their modifier sets edited.
"""

ghenv.Component.Name = 'HB Apply ModifierSet'
ghenv.Component.NickName = 'ApplyModSet'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '6'

try:  # import the honeybee extension
    from honeybee.model import Model
    from honeybee.room import Room
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance extension
    from honeybee_radiance.lib.modifiersets import modifier_set_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import the ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # duplicate the initial objects
    rooms = [obj.duplicate() for obj in _rooms]

    # extract any rooms from the input Models
    hb_objs = []
    for hb_obj in rooms:
        if isinstance(hb_obj, Model):
            hb_objs.extend(hb_obj.rooms)
        elif isinstance(hb_obj, Room):
            hb_objs.append(hb_obj)
        else:
            raise ValueError(
                'Expected Honeybee Room or Model. Got {}.'.format(type(hb_obj)))

    # process the input modifier set if it's a string
    if isinstance(_mod_set, str):
        _mod_set = modifier_set_by_identifier(_mod_set)

    # assign the modifier set
    for rm in hb_objs:
        rm.properties.radiance.modifier_set = _mod_set



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a point-in-time climate-based sky from a Wea.
-

    Args:
        north_: A number between 0 and 360 that represents the degrees off from
            the y-axis to make North. This can also be a vector to set the North.
            Default is 0. The default North direction is the Y-axis (0 degrees).
        _wea: A Ladybug Wea object.
        _month_: An integer between 1 and 12 for the month of the year (default: 6).
        _day_: An integer between 1 and 31 for the day of the month (default: 21).
        _hour_: A number between 0 and 23.999.. for the hour of the day (default: 12).
        colored_: Boolean to note whether the sky will be rendered in full
            color (True) or it will simple be a grey sky with the same average
            value as the colored sky (False). (Default: False).

    Returns:
        sky: A Honeybee sky that can be used to create a point-in-time recipe.
"""

ghenv.Component.Name = 'HB Climatebased Sky'
ghenv.Component.NickName = 'ClimateBased'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '2 :: Light Sources'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

try:
    from ladybug_geometry.geometry2d.pointvector import Vector2D
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))

try:
    from honeybee_radiance.lightsource.sky import ClimateBased
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
    from ladybug_rhino.togeometry import to_vector2d
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

import math


if all_required_inputs(ghenv.Component):
    # process the north input
    north_ = north_ or 0
    try:  # it's a vector
        north_ = math.degrees(to_vector2d(north_).angle_clockwise(Vector2D(0, 1)))
    except AttributeError:  # north angle instead of vector
        north_ = float(north_)

    # set default values
    _month_ = _month_ or 6
    _day_ = _day_ or 21
    _hour_ = 12 if _hour_ is None else _hour_

    # ensure the sun position is aligned with the Wea timestep
    if _wea.timestep == 1 and int(_hour_) == _hour_:
        _hour_ += 0.5

    # create the sky object
    sky = ClimateBased.from_wea(
        _wea, _month_, _day_, _hour_, north_, is_colored=colored_)



# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Adjust and format a High Dynamic Range (HDR) image file.
_
Possible adjustments include chaging the exposure of the image to mimic what would
be seen by a human eye and adding an optional text label to the image.
-

    Args:
        _hdr: Path to a High Dynamic Range (HDR) image file.
        adj_expos_: Boolean to note whether the exposure of the image should be adjusted to
            mimic the human visual response in the output. The goal of this process
            is to output an image that correlates more strongly with a person’s
            subjective impression of a scene rather than the absolute birghtness
            of the scene. (Default: False).
        label_: Optional text label to be appended to the bottom of the image. This
            is useful when one has several images and would like to easily
            identify them while scrolling through them.
        label_hgt_: An integer for the height of the label text in pixels. (Default: 32).

    Returns:
        hdr: Path to the resulting adjusted HDR image file.
"""

ghenv.Component.Name = 'HB Adjust HDR'
ghenv.Component.NickName = 'AdjustHDR'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

import os

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.pcond import Pcond
    from honeybee_radiance_command.psign import Psign
    from honeybee_radiance_command.pcompos import Pcompos
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.config import folders as rad_folders
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

# check the Radiance date of the installed radiance
try:  # import lbt_recipes dependencies
    from lbt_recipes.version import check_radiance_date
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))
check_radiance_date()


if all_required_inputs(ghenv.Component):
    # set up the paths for the various files used in translation
    img_dir = os.path.dirname(_hdr)
    input_image = os.path.basename(_hdr)
    hdr = _hdr
    commands = []

    # add the command to adjust the exposure to reflect human vision (if requested)
    if adj_expos_:
        adj_image = input_image.lower().replace('.hdr', '_h.HDR')
        pcond = Pcond(input=input_image, output=adj_image)
        pcond.options.h = True
        commands.append(pcond)
        hdr = os.path.join(img_dir, adj_image)
        input_image = adj_image

    # add the command to for a text label (if requested)
    if label_:
        label_images = []
        for i, l_tex in enumerate(reversed(label_.split('\n'))):
            label_image = 'label{}.HDR'.format(i)
            psign = Psign(text=l_tex, output=label_image)
            psign.options.cb = (0, 0, 0)
            psign.options.cf = (1, 1, 1)
            psign.options.h = label_hgt_ if label_hgt_ is not None else 32
            commands.append(psign)
            label_images.append(label_image)

        lbl_image = input_image.lower().replace('.hdr', '_label.HDR')
        pcompos = Pcompos(input=label_images + [input_image], output=lbl_image)
        pcompos.options.a = 1
        commands.append(pcompos)
        hdr = os.path.join(img_dir, lbl_image)
        input_image = lbl_image

    # run the commands in series and load the global horizontal irradiance
    env = None
    if rad_folders.env != {}:
        env = rad_folders.env
    env = dict(os.environ, **env) if env else None
    for r_cmd in commands:
        r_cmd.run(env, cwd=img_dir)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a ModifierSet object containing all radiance modifiers needed to create
an radiance model. ModifierSets can be assigned to Honeybee Rooms to specify
all default modifiers on the Room.
-

    Args:
        _name_: Text to set the name for the ModifierSet and to be incorporated
            into a unique ModifierSet identifier. If None, a random one will
            be genrated.
        base_mod_set_: An optional ModifierSet object that will be used
            as the starting point for the new ModifierSet output from this
            component. This can also be text for the name of a ModifierSet
            within the library such as that output from the "HB Search Modifier
            Sets" component. If None, the Honeybee "Generic Default Modifier
            Set" will be used as the base.
        _exterior_subset_: A modifier subset list from the "HB Exterior Modifier
            Subset" component. Note that None values in this list correspond to
            no change to the given modifier in the base_mod_set_.
        _interior_subset_: A modifier subset list from the "HB Interior Modifier
            Subset" component. Note that None values in this list correspond to
            no change to the given modifier in the base_mod_set_.
        _subface_subset_: A modifier subset list from the "HB Subface Subset"
            component. Note that None values in this list correspond to no
            change to the given modifier in the base_mod_set_.
        _shade_subset_: A modifier subset list from the "HB Shade Modifier
            Subset" component. Note that None values in this list correspond to
            no change to the given modifier in the base_mod_set_.

    Returns:
        mod_set: A ModifierSet object that can be assigned to Honeybee
            Rooms in order to specify all default modifiers on the Room.
"""

ghenv.Component.Name = 'HB ModifierSet'
ghenv.Component.NickName = 'ModifierSet'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_rad_string, clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.modifierset import ModifierSet
    from honeybee_radiance.lib.modifiersets import modifier_set_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import turn_off_old_tag
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))
turn_off_old_tag(ghenv.Component)


# get the base modifier set
name = clean_and_id_rad_string('ModifierSet') if _name_ is None else \
    clean_rad_string(_name_)
if base_mod_set_ is None:
    mod_set = ModifierSet(name)
else:
    if isinstance(base_mod_set_, str):
        base_mod_set_ = modifier_set_by_identifier(base_mod_set_)
    mod_set = base_mod_set_.duplicate()
    mod_set.identifier = name
    if _name_ is not None:
        mod_set.display_name = _name_

# go through each input modifier subset and assign it to the set
if len(_exterior_subset_) != 0:
    assert len(_exterior_subset_) == 3, 'Input _exterior_subset_ is not valid.'
    if _exterior_subset_[0] is not None:
        mod_set.wall_set.exterior_modifier = _exterior_subset_[0]
    if _exterior_subset_[1] is not None:
        mod_set.roof_ceiling_set.exterior_modifier = _exterior_subset_[1]
    if _exterior_subset_[2] is not None:
        mod_set.floor_set.exterior_modifier = _exterior_subset_[2]

if len(_interior_subset_) != 0:
    assert len(_interior_subset_) == 6, 'Input _interior_subset_ is not valid.'
    if _interior_subset_[0] is not None:
        mod_set.wall_set.interior_modifier = _interior_subset_[0]
    if _interior_subset_[1] is not None:
        mod_set.roof_ceiling_set.interior_modifier = _interior_subset_[1]
    if _interior_subset_[2] is not None:
        mod_set.floor_set.interior_modifier = _interior_subset_[2]
    if _interior_subset_[3] is not None:
        mod_set.aperture_set.interior_modifier = _interior_subset_[3]
    if _interior_subset_[4] is not None:
        mod_set.door_set.interior_modifier = _interior_subset_[4]
    if _interior_subset_[5] is not None:
        mod_set.door_set.interior_glass_modifier = _interior_subset_[5]

if len(_subface_subset_) != 0:
    assert len(_subface_subset_) == 6, 'Input _subface_subset_ is not valid.'
    if _subface_subset_[0] is not None:
        mod_set.aperture_set.window_modifier = _subface_subset_[0]
    if _subface_subset_[1] is not None:
        mod_set.aperture_set.skylight_modifier = _subface_subset_[1]
    if _subface_subset_[2] is not None:
        mod_set.aperture_set.operable_modifier = _subface_subset_[2]
    if _subface_subset_[3] is not None:
        mod_set.door_set.exterior_modifier = _subface_subset_[3]
    if _subface_subset_[4] is not None:
        mod_set.door_set.overhead_modifier = _subface_subset_[4]
    if _subface_subset_[5] is not None:
        mod_set.door_set.exterior_glass_modifier = _subface_subset_[5]

if len(_shade_subset_) != 0:
    assert len(_shade_subset_) == 2, 'Input _shade_subset_ is not valid.'
    if _shade_subset_[0] is not None:
        mod_set.shade_set.exterior_modifier = _shade_subset_[0]
    if _shade_subset_[1] is not None:
        mod_set.shade_set.interior_modifier = _shade_subset_[1]



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Generate SensorGrid objects from the floors of honeybee Rooms.
These SensorGrids can be used in a grid-based recipe.
-
The names of the grids will be the same as the rooms that they came from.
-

    Args:
        _rooms: A list of honeybee Rooms for which sensor grids will be generated.
            This can also be an entire Honeybee Model from which Rooms will
            be extracted.
        _grid_size: Number for the size of the grid cells.
        _dist_floor_: Number for the distance to move points from the floors of
            the input rooms. The default is 0.8 meters.
        quad_only_: Boolean to note whether meshing should be done using Rhino's
            defaults (False), which fills the entire floor geometry to the edges
            with both quad and tringulated faces, or a mesh with only quad
            faces should be generated.
            _
            FOR ADVANCED USERS: This input can also be a vector object that will
            be used to set the orientation of the quad-only grid. Note that,
            if a vector is input here that is not aligned with the plane of
            the room's floors, an error will be raised.
        remove_out_: Boolean to note whether an extra check should be run to remove
            sensor points that lie outside the Room volume. Note that this can
            add significantly to the component's run time and this check is
            usually not necessary in the case that all walls are vertical
            and all floors are horizontal (Default: False).
        wall_offset_: A number for the distance at which sensors close to walls
            should be removed.

    Returns:
        grid: A SensorGrid object that can be used in a grid-based recipe.
        points: The points that are at the center of each grid cell.
        mesh: Analysis mesh that can be passed to the 'Spatial Heatmap' component.
"""

ghenv.Component.Name = 'HB Sensor Grid from Rooms'
ghenv.Component.NickName = 'GridRooms'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

import math

try:  # import the ladybug_geometry dependencies
    from ladybug_geometry.geometry3d.plane import Plane
    from ladybug_geometry.geometry3d.face import Face3D
    from ladybug_geometry.geometry3d.mesh import Mesh3D
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))

try:  # import the core honeybee dependencies
    from honeybee.model import Model
    from honeybee.room import Room
    from honeybee.facetype import Floor, Wall
    from honeybee.typing import clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.sensorgrid import SensorGrid
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.config import conversion_to_meters, tolerance
    from ladybug_rhino.togeometry import to_joined_gridded_mesh3d, to_vector3d
    from ladybug_rhino.fromgeometry import from_mesh3d, from_point3d, from_face3d
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set defaults for any blank inputs and process the quad_only_
    _dist_floor_ = 0.8 / conversion_to_meters() if _dist_floor_ is None else _dist_floor_
    try:
        x_axis = to_vector3d(quad_only_)
    except AttributeError:
        x_axis = None

    # create lists to be filled with content
    grid = []
    points = []
    mesh = []
    rooms = []
    for obj in _rooms:
        if isinstance(obj, Model):
            rooms.extend(obj.rooms)
        elif isinstance(obj, Room):
            rooms.append(obj)
        else:
            raise TypeError('Expected Honeybee Room or Model. Got {}.'.format(type(obj)))

    for room in rooms:
        # get all of the floor faces of the room as Breps
        lb_floors = [face.geometry.flip() for face in room.faces if isinstance(face.type, Floor)]

        if len(lb_floors) != 0:
            # create the gridded ladybug Mesh3D
            if quad_only_:  # use Ladybug's built-in meshing methods
                if x_axis:
                    lb_floors = [Face3D(f.boundary, Plane(f.normal, f[0], x_axis), f.holes)
                                 for f in lb_floors]
                lb_meshes = []
                for geo in lb_floors:
                    try:
                        lb_meshes.append(geo.mesh_grid(_grid_size, offset=_dist_floor_))
                    except AssertionError:
                        continue
                if len(lb_meshes) == 0:
                    lb_mesh = None
                else:
                    lb_mesh = lb_meshes[0] if len(lb_meshes) == 1 else \
                        Mesh3D.join_meshes(lb_meshes)
            else:  # use Rhino's default meshing
                floor_faces = [from_face3d(face) for face in lb_floors]
                lb_mesh = to_joined_gridded_mesh3d(floor_faces, _grid_size, _dist_floor_)

            # remove points outside of the room volume if requested
            if remove_out_ and lb_mesh is not None:
                pattern = [room.geometry.is_point_inside(pt)
                           for pt in lb_mesh.face_centroids]
                try:
                    lb_mesh, vertex_pattern = lb_mesh.remove_faces(pattern)
                except AssertionError:  # the grid lies completely outside of the room
                    lb_mesh = None

            # remove any sensors within a certain distance of the walls, if requested
            if wall_offset_ is not None and lb_mesh is not None:
                wall_geos = [f.geometry for f in room.faces if isinstance(f.type, Wall)]
                pattern = []
                for pt in lb_mesh.face_centroids:
                    for wg in wall_geos:
                        close_pt = wg.plane.closest_point(pt)
                        p_dist = pt.distance_to_point(close_pt)
                        if p_dist <= wall_offset_:
                            close_pt_2d = wg.plane.xyz_to_xy(close_pt)
                            g_dist = wg.polygon2d.distance_to_point(close_pt_2d)
                            f_dist = math.sqrt(p_dist ** 2 + g_dist ** 2)
                            if f_dist <= wall_offset_:
                                pattern.append(False)
                                break
                    else:
                        pattern.append(True)
                try:
                    lb_mesh, vertex_pattern = lb_mesh.remove_faces(pattern)
                except AssertionError:  # the grid lies completely outside of the room
                    lb_mesh = None

            if lb_mesh is not None:
                # extract positions and directions from the mesh
                base_points = [from_point3d(pt) for pt in lb_mesh.face_centroids]
                base_poss = [(pt.x, pt.y, pt.z) for pt in lb_mesh.face_centroids]
                base_dirs = [(vec.x, vec.y, vec.z) for vec in lb_mesh.face_normals]

                # create the sensor grid
                s_grid = SensorGrid.from_position_and_direction(
                    clean_rad_string(room.display_name), base_poss, base_dirs)
                s_grid.display_name = room.display_name
                s_grid.room_identifier = room.identifier
                s_grid.mesh = lb_mesh
                s_grid.base_geometry = \
                    tuple(f.move(f.normal * _dist_floor_) for f in lb_floors)

                # append everything to the lists
                grid.append(s_grid)
                points.append(base_points)
                mesh.append(from_mesh3d(lb_mesh))

    # convert the lists of points to data trees
    points = list_to_data_tree(points)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Calculate Aperture groups for exterior Apertures.
_
The Apertures are grouped by orientation unless _view_factor_ is set to True.
_
If grouping based on view factor the component calculates view factor from
Apertures to sky patches (rfluxmtx). Each Aperture is represented by a sensor
grid, and the view factor for the whole Aperture is the average of the grid. The
RMSE of the view factor to each sky patch is calculated between all Apertures.
Agglomerative hierarchical clustering (with complete-linkage method) is used to
group the Apertures by using a distance matrix of the RMSE values.

The view factor approach is Radiance-based (and slower) and will likely group
Apertures more accurately considering the context geometry of the Honeybee
Model.
-

    Args:
        _model: A Honeybee Model for which Apertures will be grouped automatically.
            Note that this model must have Apertures with Outdoors boundary
            condition assigned to it.
        _room_based_: A boolean to note whether the Apertures should be grouped on a
            room basis. If grouped on a room basis Apertures from different
            room cannot be in the same group. (Default: True).
        _view_factor_: A boolean to note whether the Apertures should be grouped by
            calculating view factors for the Apertures to a discretized sky or
            simply by the normal orientation of the Apertures. (Default: False).
        _size_: Aperture grid size for view factor calculation. A lower number
            will give a finer grid and more accurate results but the calculation
            time will increase. This option is only used if _view_factor_ is set
            to True. (Default: 0.2).
        vert_tolerance_: A float value for vertical tolerance between two Apertures.
            If the vertical distance between two Apertures is larger than this
            tolerance the Apertures cannot be grouped. If no value is given the
            vertical grouping will be skipped. (Default: None).
        states_: An optional list of Honeybee State objects to be applied to all the generated groups.
            These states should be ordered based on how they will be switched on.
            The first state is the default state and, typically, higher states
            are more shaded. If the objects in the group have no states, the
            modifiers already assigned the apertures will be used for all states.
        _run: Set to True to run the automatic Aperture grouping.

    Returns:
        model: The input Honeybee Model object where all Apertures with Outdoors
            boundary condition have been assigned a dynamic group identifier.
"""

ghenv.Component.Name = 'HB Automatic Aperture Group'
ghenv.Component.NickName = 'AutoGroup'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

import os
import json

try:  # import honeybee_radiance dependencies
    from ladybug.futil import write_to_file_by_name
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee.model import Model
    from honeybee.boundarycondition import Outdoors
    from honeybee.config import folders
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.oconv import Oconv
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:
    from honeybee_radiance.config import folders as rad_folders
    from honeybee_radiance.dynamic.multiphase import aperture_view_factor, \
        aperture_view_factor_postprocess, cluster_view_factor, \
        cluster_orientation, cluster_output
    from honeybee_radiance.lightsource.sky.skydome import SkyDome
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run:
    assert isinstance(_model, Model), \
        'Input _model must be a Model. Got {}'.format(type(_model))
    # duplicate model
    model = _model.duplicate()

    # set defaults
    room_based = True if _room_based_ is None else _room_based_
    view_factor = False if _view_factor_ is None else _view_factor_
    size = 0.2 if _size_ is None else _size_
    vertical_tolerance = None if vert_tolerance_ is None else vert_tolerance_

    # create directory
    folder_dir = os.path.join(folders.default_simulation_folder, 'aperture_groups')
    if not os.path.isdir(folder_dir):
        os.makedirs(folder_dir)

    apertures = []
    room_apertures = {}
    # get all room-based apertures with Outdoors boundary condition
    for room in model.rooms:
        for face in room.faces:
            for ap in face.apertures:
                if isinstance(ap.boundary_condition, Outdoors):
                    apertures.append(ap)
                    if not room.identifier in room_apertures:
                        room_apertures[room.identifier] = {}
                    if not 'apertures' in room_apertures[room.identifier]:
                        room_apertures[room.identifier]['apertures'] = \
                            [ap]
                    else:
                        room_apertures[room.identifier]['apertures'].append(ap)
                    if not 'display_name' in room_apertures[room.identifier]:
                        room_apertures[room.identifier]['display_name'] = \
                            room.display_name
    assert len(apertures) != 0, \
        'Found no apertures. There should at least be one aperture ' \
        'in your model.'

    if view_factor:
        # write octree
        model_content, modifier_content = model.to.rad(model, minimal=True)
        scene_file, mat_file = 'scene.rad', 'scene.mat'
        write_to_file_by_name(folder_dir, scene_file, model_content)
        write_to_file_by_name(folder_dir, mat_file, modifier_content)
        
        octree = 'scene.oct'
        oconv = Oconv(inputs=[mat_file, scene_file], output=octree)
        oconv.options.f = True
        
        # run Oconv command
        env = None
        if rad_folders.env != {}:
            env = rad_folders.env
        env = dict(os.environ, **env) if env else None
        oconv.run(env, cwd=folder_dir)
        
        rflux_sky = SkyDome()
        rflux_sky = rflux_sky.to_file(folder_dir, name='rflux_sky.sky')
        
        # calculate view factor
        mtx_file, ap_dict = aperture_view_factor(
            folder_dir, apertures, size=size, ambient_division=1000,
            receiver=rflux_sky, octree=octree, calc_folder=folder_dir
        )
        rmse = aperture_view_factor_postprocess(
            mtx_file, ap_dict, room_apertures, room_based
        )

    # cluster apertures into groups
    if view_factor:
        ap_groups = cluster_view_factor(
            rmse, room_apertures, apertures, 0.001, room_based, vertical_tolerance)
    else:
        ap_groups = cluster_orientation(
            room_apertures, apertures, room_based, vertical_tolerance
        )

    # process clusters
    group_names, group_dict = \
        cluster_output(ap_groups, room_apertures, room_based)

    # write aperture groups to JSON file
    dyn_gr = os.path.join(folder_dir, 'aperture_groups.json')
    with open(dyn_gr, 'w') as fp:
        json.dump(group_names, fp, indent=2)

    # write dynamic group identifiers to JSON file
    dyn_gr_ids = os.path.join(folder_dir, 'dynamic_group_identifiers.json')
    with open(dyn_gr_ids, 'w') as fp:
        json.dump(group_dict, fp, indent=2)

    # assign dynamic group identifiers for each aperture
    group_ap_dict = {}
    for room in model.rooms:
        for face in room.faces:
            for ap in face.apertures:
                if isinstance(ap.boundary_condition, Outdoors):
                    dyn_group_id = group_dict[ap.identifier]
                    ap.properties.radiance.dynamic_group_identifier = \
                        dyn_group_id
                    try:
                        group_ap_dict[dyn_group_id].append(ap)
                    except KeyError:
                        group_ap_dict[dyn_group_id] = [ap]

    # assign any states if they are connected
    if len(states_) != 0:
        for group_aps in group_ap_dict.values():
            # assign states (including shades) to the first aperture
            group_aps[0].properties.radiance.states = [state.duplicate() for state in states_]
            # remove shades from following apertures to ensure they aren't double-counted
            states_wo_shades = []
            for state in states_:
                new_state = state.duplicate()
                new_state.remove_shades()
                states_wo_shades.append(new_state)
            for ap in group_aps[1:]:
                ap.properties.radiance.states = \
                    [state.duplicate() for state in states_wo_shades]



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a WEA object for an ASHRAE Revised Clear Sky (Tau Model) using a STAT file.
-

    Args:
        _stat_file: Full path to .stat file that will be used to make the clear
            sky Wea. Note that an error will be raised if no atmospheric
            optical data is found in the file. In this case, the "HB Wea from
            Clear Sky" component can be used.
        hoys_: An optional list of hours of the year (numbers from 0 to 8759) for
            which the Wea will be filtered. HOYs can be generated from the
            "LB Analysis Period" component or they can be obtained through
            other means like analysis of the values in an occupancy schedule.
            By default, the Wea will be generated for the whole year.
        timestep_: An integer representing the timestep with which to make the 
            WEA object.  Default is set to 1 for 1 step per hour of the year.

    Returns:
        wea: A wea object from stat file. This wea object represents an ASHRAE Revised 
            Clear Sky ("Tau Model"), which is intended to determine peak solar load 
            and sizing parmeters for HVAC systems. The "Tau Model" uses monthly 
            optical depths found within a .stat file.
"""

ghenv.Component.Name = 'HB Wea From Tau Clear Sky'
ghenv.Component.NickName = 'TauClearSky'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '2 :: Light Sources'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:
    from ladybug.wea import Wea
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    timestep_ = 1 if timestep_ is None else timestep_
    wea = Wea.from_stat_file(_stat_file, timestep_)
    if len(hoys_) != 0:
        wea = wea.filter_by_hoys(hoys_)


# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a list of interior modifiers that can be used to edit or create a
ModifierSet object.
-

    Args:
        _interior_wall_: A modifier object for interior walls (or text for
            the identifier of the modifier within the library).
        _ceiling_: A modifier object for ceilings (or text for the identifier of
            the modifier within the library).
        _interior_floor_: A modifier object for interior floors (or text for
            the identifier of the modifier within the library).
        _interior_window_: A modifier object for all apertures with a Surface
            boundary condition. This can also be text for the identifier of the
            modifier within the library.
        _interior_door_: A modifier object for all opaque doors with a Surface
            boundary condition. This can also be text for the identifier of the
            modifier within the library.
        _int_glass_door_: A modifier object for all glass doors with a Surface
            boundary condition. This can also be text for the identifier of the
            modifier within the library.
    
    Returns:
        interior_set: A list of interior modifiers that can be used to edit
            or create a ModifierSet object.
"""

ghenv.Component.Name = 'HB Interior Modifier Subset'
ghenv.Component.NickName = 'InteriorSubset'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.modifier import Modifier
    from honeybee_radiance.lib.modifiers import modifier_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import turn_off_old_tag
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))
turn_off_old_tag(ghenv.Component)


def check_mod(mod, input_name):
    """Get an Modifier from the library if it's a string."""
    if isinstance(mod, str):
        return modifier_by_identifier(mod)
    else:
        assert isinstance(mod, Modifier), \
            'Expected Modifier for {}. Got {}'.format(input_name, type(mod))
    return mod


# go through each input modifier
if _interior_wall_ is not None:
    _interior_wall_ = check_mod(_interior_wall_, '_interior_wall_')
if _ceiling_ is not None:
    _ceiling_ = check_mod(_ceiling_, '_ceiling_')
if _interior_floor_ is not None:
    _interior_floor_ = check_mod(_interior_floor_, '_interior_floor_')
if _interior_window_ is not None:
    _interior_window_ = check_mod(_interior_window_, '_interior_window_')
if _interior_door_ is not None:
    _interior_door_ = check_mod(_interior_door_, '_interior_door_')
if _int_glass_door_ is not None:
    _int_glass_door_ = check_mod(_int_glass_door_, '_int_glass_door_')

# return the final list from the component
interior_set = [_interior_wall_, _ceiling_, _interior_floor_, _interior_window_,
                _interior_door_, _int_glass_door_]



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a Honeybee View for an image-based analysis using a Rhino viewport.
-

    Args:
        _name_: Text to set the name for the modifier and to be incorporated into
            a unique modifier identifier.
        _viewport_: The Rhino viewport name which will be used to generate a radiance
            View object. Typical inputs include "Perspective", "Top", "Bottom",
            "Left", "Right", "Front", "Back" or any viewport name that you have
            saved within the Rhino file.  If no text is input here, the default
            will be the currently active viewport (the last viewport in which
            you navigated).
        _view_type_: An integer to set the view type (-vt). Choose from the choices
            below. Default: 0 if the viewport is in perspective; 2 if it is parallel.
                * 0 Perspective (v)
                * 1 Hemispherical fisheye (h)
                * 2 Parallel (l)
                * 3 Cylindrical panorama (c)
                * 4 Angular fisheye (a)
                * 5 Planisphere [stereographic] projection (s)
            For more detailed description about view types check rpict manual
            page (http://radsite.lbl.gov/radiance/man_html/rpict.1.html)
        refresh_: Connect a Grasshopper "button" component to refresh the orientation
            upon hitting the button.

    Returns:
        view: A Honeybee View object that can be used in a view-based recipe.
"""

ghenv.Component.Name = 'HB View from Viewport'
ghenv.Component.NickName = 'Viewport'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_rad_string, clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_radiance.view import View
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import turn_off_old_tag
    from ladybug_rhino.viewport import viewport_by_name, viewport_properties
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))
turn_off_old_tag(ghenv.Component)

VIEW_TYPES = ('v', 'h', 'l', 'c', 'a', 's')


# set the default values
name = clean_and_id_rad_string('View') if _name_ is None else _name_
if _view_type_ is None:
    _type_= 0
else:
    _type_= VIEW_TYPES.index(_view_type_) if _view_type_ in VIEW_TYPES else int(_view_type_)

# process the Rhino viewport
viewp = viewport_by_name(_viewport_)
v_props = viewport_properties(viewp, _type_)

# create the view object
view = View(
    clean_rad_string(name), v_props['position'], v_props['direction'],
    v_props['up_vector'], VIEW_TYPES[v_props['view_type']],
    v_props['h_angle'], v_props['v_angle'])
view.standardize_fisheye()
if _name_ is not None:
    view.display_name = _name_



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Run a quick view-based Radiance simulation to visualize the properties of Honeybee
objects within Radiance.
_
Note that this simulation is always run on a single processor and will only show
static Radiance properties (no dynamic Aperture or Shade properties). Accordingly, this
component is only intended for quick checks of properties. For full customization
of view-based simulations, the "HB Point-in-time View-based" recipe should be used.

-
    Args:
        _hb_objs: An array of honeybee Rooms, Faces, Apertures, Doors or Shades to be
            visualized in Radiance. This can also be an entire Model to be
            visualized.
        _view_: An optional Honeybee-Radiance view to specify the view to render. If
            unspecified, the currently active Rhino viewport will be rendered.
        _sky_: An optional Radiance sky from any of the sky components under the "Light
            Sources" tab. If unspecified, a uniform sky with 10000 lux will be used.
        adj_expos_: Boolean to note whether the exposure of the image should be adjusted to
            mimic the human visual response in the output. The goal of this process
            is to output an image that correlates more strongly with a person’s
            subjective impression of a scene rather than the absolute birghtness
            of the scene. (Default: True).
        _metric_: Either an integer or the full name of a point-in-time metric to be
            computed by the recipe. (Default: luminance). Choose from the following:
                * 0 = illuminance
                * 1 = irradiance
                * 2 = luminance
                * 3 = radiance
        _resolution_: An integer for the maximum dimension of each image in pixels
            (either width or height depending on the input view angle and
            type). (Default: 800).
        radiance_par_: Text for the radiance parameters to be used for ray
            tracing. (Default: -ab 2 -aa 0.25 -ad 512 -ar 16).
        _run: Set to "True" to run Radiance and get an image of the scene.

    Returns:
        report: Reports, errors, warnings, etc.
        hdr: A High Dynamic Range (HDR) image of the scene. This can be plugged into
            the Ladybug "Image Viewer" component to preview the image. It 
            can also be plugged into the "HB False Color" component to convert 
            the image into a false color version. Lastly, it can be connected to 
            the "HB HDR to GIF" component to get a GIF image that is more portable 
            and easily previewed by different software. Pixel values are in the
            standard SI units of the requested input metric.
                * illuminance = lux (aka. lm/m2)
                * irradiance = W/m2
                * luminance = cd/m2 (aka. lm/m2-sr)
                * radiance = W/m2-sr
"""

ghenv.Component.Name = 'HB Check Scene'
ghenv.Component.NickName = 'CheckScene'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '6'

import os

try:  # import honeybee_radiance dependencies
    from ladybug.futil import write_to_file_by_name
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:  # import honeybee dependencies
    from honeybee.config import folders
    from honeybee.model import Model
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import honeybee_radiance_command dependencies
    from honeybee_radiance_command.oconv import Oconv
    from honeybee_radiance_command.rpict import Rpict
    from honeybee_radiance_command.rtrace import Rtrace
    from honeybee_radiance_command.rcalc import Rcalc
    from honeybee_radiance_command.pcond import Pcond
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance_command:\n\t{}'.format(e))

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.config import folders as rad_folders
    from honeybee_radiance.view import View
    from honeybee_radiance.lightsource.sky.strutil import string_to_sky
    from honeybee_radiance.lightsource.sky import CertainIrradiance, ClimateBased
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
    from ladybug_rhino.viewport import viewport_by_name, viewport_properties
    from ladybug_rhino.config import tolerance, angle_tolerance, units_system
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))

# check the Radiance date of the installed radiance
try:  # import lbt_recipes dependencies
    from lbt_recipes.version import check_radiance_date
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))
check_radiance_date()

# dictionary of supported metrics
metric_dict = {
    '0': 'illuminance',
    '1': 'irradiance',
    '2': 'luminance',
    '3': 'radiance',
    'illuminance': 'illuminance',
    'irradiance': 'irradiance',
    'luminance': 'luminance',
    'radiance': 'radiance'
}


if all_required_inputs(ghenv.Component) and _run:
    # set defaults for resolution, metric and view
    _resolution_ = 800 if _resolution_ is None else _resolution_
    try:
        _metric_ = metric_dict[_metric_.lower()] if _metric_ is not None else 'luminance'
    except KeyError:
        raise ValueError('Metric "{}" is not supported.'.format(_metric_))
    if _view_ is None:
        viewp = viewport_by_name(None)
        v_props = viewport_properties(viewp, 0)
        VIEW_TYPES = ('v', 'h', 'l', 'c', 'a')
        _view_ = View(
            'current_viewport', v_props['position'], v_props['direction'],
            v_props['up_vector'], VIEW_TYPES[v_props['view_type']],
            v_props['h_angle'], v_props['v_angle'])
    else:
        assert isinstance(_view_, View), 'Expected Radiance View. Got {}.'.format(type(_view_))

    # process the sky input
    if _sky_ is None:
        _sky_ = CertainIrradiance.from_illuminance(10000)
    elif isinstance(_sky_, str):  # convert the sky string into a sky object
        _sky_ = string_to_sky(_sky_)
    to_rad_int = 1 if _metric_ in ('irradiance', 'radiance') else 0
    sky_content = _sky_.to_radiance(to_rad_int) if isinstance(_sky_, ClimateBased) \
        else _sky_.to_radiance()

    # process the _hb_objs into a Model and then a Radiance string
    models = [obj for obj in _hb_objs if isinstance(obj, Model)]
    other_objs = [obj for obj in _hb_objs if not isinstance(obj, Model)]
    model = Model.from_objects('scene', other_objs, units_system(), tolerance, angle_tolerance)
    for m in models:
        model.add_model(m)
    model_content, modifier_content = model.to.rad(model, minimal=True)

    # set up the paths for the various files used in translation
    scene_dir = os.path.join(folders.default_simulation_folder, 'scene_visualiztion')
    sky_file, scene_file, mat_file = \
        'weather.sky', 'scene.rad', 'scene.mat'
    view_file = 'view_{}.vf'.format(_metric_)
    write_to_file_by_name(scene_dir, sky_file, sky_content, mkdir=True)
    write_to_file_by_name(scene_dir, scene_file, model_content)
    write_to_file_by_name(scene_dir, mat_file, modifier_content)
    _view_.to_file(scene_dir, view_file)
    scene_oct, final_hdr = 'scene_visual.oct', 'scene.HDR'
    hdr = os.path.join(scene_dir, final_hdr)
    if os.path.isfile(hdr):
        os.remove(hdr)

    # build up the commands to render the image of the sky
    oconv = Oconv(inputs=[sky_file, mat_file, scene_file], output=scene_oct)
    oconv.options.f = True

    rpict = Rpict(octree=scene_oct, output=final_hdr, view=view_file)
    rpict.options.ab = 2
    rpict.options.aa = 0.25
    rpict.options.ad = 512
    rpict.options.ar = 16
    if radiance_par_:
        rpict.options.update_from_string(radiance_par_.strip())
    if _metric_ in ('illuminance', 'irradiance'):
        rpict.options.i = True
    else:
        rpict.options.i = False
    rpict.options.x = _resolution_
    rpict.options.y = _resolution_

    commands = [oconv, rpict]
    if adj_expos_ or adj_expos_ is None:
        adj_image = final_hdr.lower().replace('.hdr', '_h.HDR')
        pcond = Pcond(input=final_hdr, output=adj_image)
        pcond.options.h = True
        commands.append(pcond)
        hdr = os.path.join(scene_dir, adj_image)
        if os.path.isfile(hdr):
            os.remove(hdr)

    # run the commands in series and load the global horizontal irradiance
    env = None
    if rad_folders.env != {}:
        env = rad_folders.env
    env = dict(os.environ, **env) if env else None
    for r_cmd in commands:
        r_cmd.run(env, cwd=scene_dir)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Get all of the Dynamic Radiance Groups assigned to a Model.
-

    Args:
        _model: A Honeybee Model for which dynamic groups will be output.

    Returns:
        group_ids: The identifiers of the dynamic groups assigned to the Model.
        group_aps: A data tree of Dynamic Apertures in the Model. Each branch of the
            tree represents a different Dynamic Aperture Group and corresponds to
            the group_ids above. The data tree can be exploded with the native
            Grasshopper "Explod Tree" component to assign schedules to each
            Dynamic Group for postprocessing.
"""

ghenv.Component.Name = 'HB Get Dynamic Groups'
ghenv.Component.NickName = 'GetDyn'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '0'

try:  # import core honeybee dependencies
    from honeybee.model import Model
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs, list_to_data_tree
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    assert isinstance(_model, Model), \
        'Expected Honeybee Model. Got {}.'.format(type(_model))
    # get the dynamic group objects
    groups = _model.properties.radiance.dynamic_subface_groups
    groups.sort(key=lambda x: x.identifier)

    # get the group attributes
    group_ids, group_aps = [], []
    for group in groups:
        group_ids.append([group.identifier])
        group_aps.append(group.dynamic_objects)
    group_ids = list_to_data_tree(group_ids)
    group_aps = list_to_data_tree(group_aps)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a metal radiance modifier from red, green, and blue reflectances.
-

    Args:
        _name_: Text to set the name for the modifier and to be incorporated into
            a unique modifier identifier.
        _r_diff_: A number between 0 and 1 for the absolute diffuse red reflectance. (Default: 0).
        _g_diff_: A number between 0 and 1 for the absolute diffuse green reflectance. (Default: 0).
        _b_diff_: A number between 0 and 1 for the absolute diffuse blue reflectance. (Default: 0).
        _spec_:  A number between 0 and 1 for the absolute specular reflectance of the modifier.
            Note that the sum of this value and the diffuse should be less
            than one. Specularity of metals is usually 0.9 or greater. (Default: 0.9)
        _rough_: Roughness is specified as the rms slope of surface facets. A value
            of 0 corresponds to a perfectly smooth surface, and a value of 1 would be
            a very rough surface. Roughness values greater than 0.2 are not very
            realistic. (Default: 0).

    Returns:
        modifier: A metal modifier that can be assigned to a Honeybee geometry
            or Modifier Sets.
"""

ghenv.Component.Name = 'HB Metal Modifier 3'
ghenv.Component.NickName = 'MetalMod3'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '0'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_rad_string, clean_rad_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:  # import the honeybee-radiance dependencies
    from honeybee_radiance.modifier.material import Metal
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:  # import ladybug_rhino dependencies
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set the default modifier properties
    _r_diff_ = 0 if _r_diff_ is None else _r_diff_
    _g_diff_ = 0 if _g_diff_ is None else _g_diff_
    _b_diff_ = 0 if _b_diff_ is None else _b_diff_
    _spec_ = 0.9 if _spec_ is None else _spec_
    _rough_ = 0.0 if _rough_ is None else _rough_
    name = clean_and_id_rad_string('MetalMaterial') if _name_ is None else \
        clean_rad_string(_name_)

    # create the modifier
    modifier = Metal.from_reflected_specularity(
        name, _r_diff_, _g_diff_, _b_diff_, _spec_, _rough_)
    if _name_ is not None:
        modifier.display_name = _name_



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Calculate the number of hours of direct sun received by grids of sensors in a
Honeybee model.
_
The fundamental calculation of this recipe is the same as that of the "LB Direct
Sun Hours" component except that this recipe uses Radiance, which allows the
simulation to scale better for large numbers of sensors.

-
    Args:
        _model: A Honeybee Model for which Direct Sun Hours will be simulated.
            Note that this model should have grids assigned to it in order
            to produce meaningfule results.
        _wea: A Wea object produced from the Wea components that are under the Light
            Sources tab. This can also be the path to a .wea or a .epw file.
        _timestep_: An integer for the timestep of the inpput _wea. This value will
            be used to ensure the units of the results are in hours. (Default: 1)
        north_: A number between -360 and 360 for the counterclockwise difference
            between the North and the positive Y-axis in degrees. This can
            also be Vector for the direction to North. (Default: 0).
        grid_filter_: Text for a grid identifer or a pattern to filter the sensor grids of
            the model that are simulated. For instance, first_floor_* will simulate
            only the sensor grids that have an identifier that starts with
            first_floor_. By default, all grids in the model will be simulated.
        run_settings_: Settings from the "HB Recipe Settings" component that specify
            how the recipe should be run. This can also be a text string of
            recipe settings.
        _run: Set to True to run the recipe and get results. This input can also be
            the integer "2" to run the recipe silently.

    Returns:
        report: Reports, errors, warnings, etc.
        results: Raw result files (.ill) that contain matrices of zero/one values
            indicating whether each sensor is exposed to the sun at a given
            time step of the input Wea.
        hours: The cumulative number of hours that each sensor can see the sun.
            Each value is always in hours provided that the input _timestep_
            is the same as the input Wea.
"""

ghenv.Component.Name = 'HB Direct Sun Hours'
ghenv.Component.NickName = 'DirectSunHours'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from lbt_recipes.recipe import Recipe
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run:
    # create the recipe and set the input arguments
    recipe = Recipe('direct-sun-hours')
    recipe.input_value_by_name('model', _model)
    recipe.input_value_by_name('wea', _wea)
    recipe.input_value_by_name('timestep', _timestep_)
    recipe.input_value_by_name('north', north_)
    recipe.input_value_by_name('grid-filter', grid_filter_)

    # run the recipe
    silent = True if _run > 1 else False
    project_folder = recipe.run(run_settings_, radiance_check=True, silent=silent)

    # load the results
    try:
        results = recipe_result(recipe.output_value_by_name(
            'direct-sun-hours', project_folder))
        hours = recipe_result(recipe.output_value_by_name(
            'cumulative-sun-hours', project_folder))
    except Exception:
        raise Exception(recipe.failure_message(project_folder))



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a list of exterior subface (apertures + doors) modifiers that can be
used to edit or create a ModifierSet object.
-

    Args:
        _window_: A modifier object for apertures with an Outdoors boundary
            condition and a Wall face type for their parent face. This can also
            be text for the identifier of the modifier within the library.
        _skylight_: A modifier object for apertures with an Outdoors boundary
            condition and a RoofCeiling or Floor face type for their parent face.
            This can also be text for the identifier of the modifier within
            the library.
        _operable_: A modifier object for apertures with an Outdoors boundary
            condition and True is_operable property. This can also be text for
            the identifier of the modifier within the library.
        _exterior_door_: A modifier object for opaque doors with an Outdoors
            boundary condition and a Wall face type for their parent face. This
            can also be text for the identifier of the modifier within
            the library.
        _overhead_door_: A modifier object for opaque doors with an Outdoors
            boundary condition and a RoofCeiling or Floor face type for their
            parent face. This can also be text for the identifier of the modifier
            within the library.
        _glass_door_: A modifier object for all glass doors with an Outdoors
            boundary condition. This can also be text for the identifier of the
            modifier within the library.
    
    Returns:
        subface_set: A list of exterior subface modifiers that can be used
            to edit or create a ModifierSet object.
"""

ghenv.Component.Name = 'HB Subface Modifier Subset'
ghenv.Component.NickName = 'SubfaceSubset'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '1 :: Modifiers'
ghenv.Component.AdditionalHelpFromDocStrings = '4'

try:  # import honeybee_radiance dependencies
    from honeybee_radiance.modifier import Modifier
    from honeybee_radiance.lib.modifiers import modifier_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import turn_off_old_tag
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))
turn_off_old_tag(ghenv.Component)


def check_mod(mod, input_name):
    """Get an Modifier from the library if it's a string."""
    if isinstance(mod, str):
        return modifier_by_identifier(mod)
    else:
        assert isinstance(mod, Modifier), \
            'Expected Modifier for {}. Got {}'.format(input_name, type(mod))
    return mod


# go through each input construction
if _window_ is not None:
    _window_ = check_mod(_window_, '_window_')
if _skylight_ is not None:
    _skylight_ = check_mod(_skylight_, '_skylight_')
if _operable_ is not None:
    _operable_ = check_mod(_operable_, '_operable_')
if _exterior_door_ is not None:
    _exterior_door_ = check_mod(_exterior_door_, '_exterior_door_')
if _overhead_door_ is not None:
    _overhead_door_ = check_mod(_overhead_door_, '_overhead_door_')
if _glass_door_ is not None:
    _glass_door_ = check_mod(_glass_door_, '_glass_door_')


# return the final list from the component
subface_set = [_window_, _skylight_, _operable_, _exterior_door_,
               _overhead_door_, _glass_door_]



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Create a Dynamic Aperture Group Schedule, which can be used to process any dynamic
aperture geometry that was run in an annual simulation.

-
    Args:
        _group_aps: Honeybee Apertures that are a part of the same dynamic group and will
            be assigned the same schedule for postprocessing. Typically, this is
            the output of the "HB Dynamic Aperture Group" component but it can
            also be the output of the "HB Get Dynamic Groups" component, which
            returns all of the dynamic groups on a particular Model.
        _schedule: A list of 8760 integers refering to the index of the aperture group state
            to be used at each hour of the simulation. This can also be a single integer
            for a static state to be used for the entire period of the simulation
            or a pattern of integers that is less than 8760 in length and will be
            repeated until the 8760 is reached. Note that 0 refers to the first
            state, 1 refers to the second state, and so on. -1 can be used to
            completely discout the aperture from the simulation for a given hour.

    Returns:
        dyn_sch: A dynamic schedule object for the input aperture group, which can be plugged
            into any of the Results components with a syn_sch input.
"""

ghenv.Component.Name = 'HB Aperture Group Schedule'
ghenv.Component.NickName = 'GroupSch'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '4 :: Results'
ghenv.Component.AdditionalHelpFromDocStrings = '1'

try:
    from honeybee.aperture import Aperture
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_radiance_postprocess.dynamic import ApertureGroupSchedule
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_radiance:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    dyn_sch = []
    dyn_ids = set()
    for ap in _group_aps:
        assert isinstance(ap, Aperture), 'Expected Aperture. Got {}.'.format(type(ap))
        dyn_grp_id = ap.properties.radiance.dynamic_group_identifier
        if dyn_grp_id is None:
            raise ValueError(
                'Input Aperture "{}" is not a part of a dynamic group.'.format(ap.display_name))
        if dyn_grp_id not in dyn_ids:
            dyn_ids.add(dyn_grp_id)
            _ap_group_sch = ApertureGroupSchedule(dyn_grp_id, _schedule)
            dyn_sch.append(_ap_group_sch)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2024, Ladybug Tools.
# You should have received a copy of the GNU Affero General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license AGPL-3.0-or-later <https://spdx.org/licenses/AGPL-3.0-or-later>

"""
Run a daylight factor study for a Honeybee model.
_
Daylight Factor (DF) is defined as the ratio of the indoor daylight illuminance
to outdoor illuminance under an unobstructed overcast sky. It is expressed as a
percentage between 0 and 100.
_
Because daylight factor is computed using an overcast sky, it does not change
with [North, East, South, West] orientation. As such, it is more suited to
assessing daylight in climates where cloudy conditions are common. The "HB
Annual Daylight" recipe yields a much more accurate assessment of daylight
and is suitable for all climates, though it requires a significantly longer
calculation time than Daylight Factor.

-
    Args:
        _model: A Honeybee Model for which Daylight Factor will be simulated.
            Note that this model should have grids assigned to it in order
            to produce meaningfule results.
        grid_filter_: Text for a grid identifer or a pattern to filter the sensor grids of
            the model that are simulated. For instance, `first_floor_*` will simulate
            only the sensor grids that have an identifier that starts with
            `first_floor_`. By default, all grids in the model will be simulated.
        radiance_par_: Text for the radiance parameters to be used for ray
            tracing. (Default: -ab 2 -aa 0.1 -ad 2048 -ar 64).
        run_settings_: Settings from the "HB Recipe Settings" component that specify
            how the recipe should be run. This can also be a text string of
            recipe settings.
        _run: Set to True to run the recipe and get results. This input can also be
            the integer "2" to run the recipe silently.

    Returns:
        report: Reports, errors, warnings, etc.
        results: The daylight factor values from the simulation in percent. Each
            value is for a different sensor of the grid. These can be plugged
            into the "LB Spatial Heatmap" component along with meshes of the
            sensor grids to visualize results.
"""

ghenv.Component.Name = 'HB Daylight Factor'
ghenv.Component.NickName = 'DaylightFactor'
ghenv.Component.Message = '1.8.0'
ghenv.Component.Category = 'HB-Radiance'
ghenv.Component.SubCategory = '3 :: Recipes'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from lbt_recipes.recipe import Recipe
except ImportError as e:
    raise ImportError('\nFailed to import lbt_recipes:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, recipe_result
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component) and _run:
    # create the recipe and set the input arguments
    recipe = Recipe('daylight-factor')
    recipe.input_value_by_name('model', _model)
    recipe.input_value_by_name('grid-filter', grid_filter_)
    recipe.input_value_by_name('radiance-parameters', radiance_par_)

    # run the recipe
    silent = True if _run > 1 else False
    project_folder = recipe.run(run_settings_, radiance_check=True, silent=silent)

    # load the results
    try:
        results = recipe_result(recipe.output_value_by_name('results', project_folder))
    except Exception:
        raise Exception(recipe.failure_message(project_folder))
